MODULE icedyn_adv_pra 
   !!======================================================================
   !!                       ***  MODULE icedyn_adv_pra   ***
   !!   sea-ice : advection => Prather scheme
   !!======================================================================
   !! History :  LIM  ! 2008-03 (M. Vancoppenolle)  LIM-3 from LIM-2 code
   !!            3.2  ! 2009-06 (F. Dupont)  correct a error in the North fold b.c.
   !!            4.0  ! 2011-02 (G. Madec) dynamical allocation
   !!--------------------------------------------------------------------
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3'                                       ESIM sea-ice model
   !!----------------------------------------------------------------------
   !!   ice_dyn_adv_pra   : advection of sea ice using Prather scheme
   !!----------------------------------------------------------------------
   USE dom_oce        ! ocean domain
   USE ice            ! sea-ice variables
   USE sbc_oce , ONLY : nn_fsbc   ! frequency of sea-ice call
   !
   USE in_out_manager ! I/O manager
   USE iom            ! I/O manager library
   USE lib_mpp        ! MPP library
   USE lib_fortran    ! fortran utilities (glob_sum + no signed zero)
   USE lbclnk         ! lateral boundary conditions (or mpp links)
   USE prtctl         ! Print control

   IMPLICIT NONE
   PRIVATE

   PUBLIC   ice_dyn_adv_pra   ! called by icedyn_adv
   PUBLIC   adv_pra_init      ! called by icedyn_adv

   ! Moments for advection
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:)     ::   sxopw, syopw, sxxopw, syyopw, sxyopw   ! open water in sea ice
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxice, syice, sxxice, syyice, sxyice   ! ice thickness 
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxsn , sysn , sxxsn , syysn , sxysn    ! snow thickness
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxa  , sya  , sxxa  , syya  , sxya     ! lead fraction
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxc0 , syc0 , sxxc0 , syyc0 , sxyc0    ! snow thermal content
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxsal, sysal, sxxsal, syysal, sxysal   ! ice salinity
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxage, syage, sxxage, syyage, sxyage   ! ice age
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxap , syap , sxxap , syyap , sxyap    ! melt pond fraction
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   ::   sxvp , syvp , sxxvp , syyvp , sxyvp    ! melt pond volume
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) ::   sxe  , sye  , sxxe  , syye  , sxye     ! ice layers heat content
   !
   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/ICE 4.0 , NEMO Consortium (2017)
   !! $Id: icedyn_adv_pra.F90 6746 2016-06-27 17:20:57Z clem $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE ice_dyn_adv_pra( kt, pu_ice, pv_ice,  &
      &                        pato_i, pv_i, pv_s, psv_i, poa_i, pa_i, pa_ip, pv_ip, pe_s, pe_i )
      !!----------------------------------------------------------------------
      !!                **  routine ice_dyn_adv_pra  **
      !!  
      !! ** purpose :   Computes and adds the advection trend to sea-ice
      !!
      !! ** method  :   Uses Prather second order scheme that advects tracers
      !!                but also their quadratic forms. The method preserves
      !!                tracer structures by conserving second order moments.
      !!
      !! Reference:  Prather, 1986, JGR, 91, D6. 6671-6681.
      !!----------------------------------------------------------------------
      INTEGER                     , INTENT(in   ) ::   kt         ! time step
      REAL(wp), DIMENSION(:,:)    , INTENT(in   ) ::   pu_ice     ! ice i-velocity
      REAL(wp), DIMENSION(:,:)    , INTENT(in   ) ::   pv_ice     ! ice j-velocity
      REAL(wp), DIMENSION(:,:)    , INTENT(inout) ::   pato_i     ! open water area
      REAL(wp), DIMENSION(:,:,:)  , INTENT(inout) ::   pv_i       ! ice volume
      REAL(wp), DIMENSION(:,:,:)  , INTENT(inout) ::   pv_s       ! snw volume
      REAL(wp), DIMENSION(:,:,:)  , INTENT(inout) ::   psv_i      ! salt content
      REAL(wp), DIMENSION(:,:,:)  , INTENT(inout) ::   poa_i      ! age content
      REAL(wp), DIMENSION(:,:,:)  , INTENT(inout) ::   pa_i       ! ice concentration
      REAL(wp), DIMENSION(:,:,:)  , INTENT(inout) ::   pa_ip      ! melt pond fraction
      REAL(wp), DIMENSION(:,:,:)  , INTENT(inout) ::   pv_ip      ! melt pond volume
      REAL(wp), DIMENSION(:,:,:,:), INTENT(inout) ::   pe_s       ! snw heat content
      REAL(wp), DIMENSION(:,:,:,:), INTENT(inout) ::   pe_i       ! ice heat content
      !
      INTEGER  ::   jk, jl, jt              ! dummy loop indices
      INTEGER  ::   initad                  ! number of sub-timestep for the advection
      REAL(wp) ::   zcfl , zusnit           !   -      -
      REAL(wp), ALLOCATABLE, DIMENSION(:,:)     ::   zarea
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:)   ::   z0opw
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:)   ::   z0ice, z0snw, z0ai, z0es , z0smi , z0oi
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:)   ::   z0ap , z0vp
      REAL(wp), ALLOCATABLE, DIMENSION(:,:,:,:) ::   z0ei
      !!----------------------------------------------------------------------
      !
      IF( kt == nit000 .AND. lwp )   WRITE(numout,*) '-- ice_dyn_adv_pra: Prather advection scheme'
      !
      ALLOCATE( zarea(jpi,jpj)     , z0opw(jpi,jpj, 1 ) ,                                           &
         &      z0ice(jpi,jpj,jpl) , z0snw(jpi,jpj,jpl) , z0ai(jpi,jpj,jpl) , z0es(jpi,jpj,jpl) ,   &
         &      z0smi(jpi,jpj,jpl) , z0oi (jpi,jpj,jpl) , z0ap(jpi,jpj,jpl) , z0vp(jpi,jpj,jpl) ,   &
         &      z0ei (jpi,jpj,nlay_i,jpl)                                                           )
      !
      ! --- If ice drift field is too fast, use an appropriate time step for advection (CFL test for stability) --- !        
      zcfl  =            MAXVAL( ABS( pu_ice(:,:) ) * rdt_ice * r1_e1u(:,:) )
      zcfl  = MAX( zcfl, MAXVAL( ABS( pv_ice(:,:) ) * rdt_ice * r1_e2v(:,:) ) )
      IF( lk_mpp )   CALL mpp_max( zcfl )
      
      IF( zcfl > 0.5 ) THEN   ;   initad = 2   ;   zusnit = 0.5_wp
      ELSE                    ;   initad = 1   ;   zusnit = 1.0_wp
      ENDIF
      
      zarea(:,:) = e1e2t(:,:)
      !-------------------------
      ! transported fields                                        
      !-------------------------
      z0opw(:,:,1) = pato_i(:,:) * e1e2t(:,:)              ! Open water area 
      DO jl = 1, jpl
         z0snw(:,:,jl) = pv_s (:,:,  jl) * e1e2t(:,:)     ! Snow volume
         z0ice(:,:,jl) = pv_i (:,:,  jl) * e1e2t(:,:)     ! Ice  volume
         z0ai (:,:,jl) = pa_i (:,:,  jl) * e1e2t(:,:)     ! Ice area
         z0smi(:,:,jl) = psv_i(:,:,  jl) * e1e2t(:,:)     ! Salt content
         z0oi (:,:,jl) = poa_i(:,:,  jl) * e1e2t(:,:)     ! Age content
         z0es (:,:,jl) = pe_s (:,:,1,jl) * e1e2t(:,:)     ! Snow heat content
         DO jk = 1, nlay_i
            z0ei(:,:,jk,jl) = pe_i(:,:,jk,jl) * e1e2t(:,:) ! Ice  heat content
         END DO
         IF ( nn_pnd_scheme > 0 ) THEN
            z0ap(:,:,jl)  = pa_ip(:,:,jl) * e1e2t(:,:)     ! Melt pond fraction
            z0vp(:,:,jl)  = pv_ip(:,:,jl) * e1e2t(:,:)     ! Melt pond volume
         ENDIF
      END DO

      !                                                    !--------------------------------------------!
      IF( MOD( ( kt - 1) / nn_fsbc , 2 ) == 0 ) THEN       !==  odd ice time step:  adv_x then adv_y  ==!
         !                                                 !--------------------------------------------!
         DO jt = 1, initad
            CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0opw (:,:,1), sxopw(:,:),   &             !--- ice open water area
               &                                      sxxopw(:,:)  , syopw(:,:), syyopw(:,:), sxyopw(:,:)  )
            CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0opw (:,:,1), sxopw(:,:),   &
               &                                      sxxopw(:,:)  , syopw(:,:), syyopw(:,:), sxyopw(:,:)  )
            DO jl = 1, jpl
               CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0ice (:,:,jl), sxice(:,:,jl),   &    !--- ice volume  ---
                  &                                      sxxice(:,:,jl), syice(:,:,jl), syyice(:,:,jl), sxyice(:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0ice (:,:,jl), sxice(:,:,jl),   &
                  &                                      sxxice(:,:,jl), syice(:,:,jl), syyice(:,:,jl), sxyice(:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0snw (:,:,jl), sxsn (:,:,jl),   &    !--- snow volume  ---
                  &                                      sxxsn (:,:,jl), sysn (:,:,jl), syysn (:,:,jl), sxysn (:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0snw (:,:,jl), sxsn (:,:,jl),   &
                  &                                      sxxsn (:,:,jl), sysn (:,:,jl), syysn (:,:,jl), sxysn (:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0smi (:,:,jl), sxsal(:,:,jl),   &    !--- ice salinity ---
                  &                                      sxxsal(:,:,jl), sysal(:,:,jl), syysal(:,:,jl), sxysal(:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0smi (:,:,jl), sxsal(:,:,jl),   &
                  &                                      sxxsal(:,:,jl), sysal(:,:,jl), syysal(:,:,jl), sxysal(:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0oi  (:,:,jl), sxage(:,:,jl),   &    !--- ice age      ---     
                  &                                      sxxage(:,:,jl), syage(:,:,jl), syyage(:,:,jl), sxyage(:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0oi  (:,:,jl), sxage(:,:,jl),   &
                  &                                      sxxage(:,:,jl), syage(:,:,jl), syyage(:,:,jl), sxyage(:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0ai  (:,:,jl), sxa  (:,:,jl),   &    !--- ice concentrations ---
                  &                                      sxxa  (:,:,jl), sya  (:,:,jl), syya  (:,:,jl), sxya  (:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0ai  (:,:,jl), sxa  (:,:,jl),   & 
                  &                                      sxxa  (:,:,jl), sya  (:,:,jl), syya  (:,:,jl), sxya  (:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0es  (:,:,jl), sxc0 (:,:,jl),   &    !--- snow heat contents ---
                  &                                      sxxc0 (:,:,jl), syc0 (:,:,jl), syyc0 (:,:,jl), sxyc0 (:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0es  (:,:,jl), sxc0 (:,:,jl),   &
                  &                                      sxxc0 (:,:,jl), syc0 (:,:,jl), syyc0 (:,:,jl), sxyc0 (:,:,jl)  )
               DO jk = 1, nlay_i                                                               !--- ice heat contents ---
                  CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0ei(:,:,jk,jl), sxe (:,:,jk,jl),   & 
                     &                                      sxxe(:,:,jk,jl), sye (:,:,jk,jl),   &
                     &                                      syye(:,:,jk,jl), sxye(:,:,jk,jl) )
                  CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0ei(:,:,jk,jl), sxe (:,:,jk,jl),   & 
                     &                                      sxxe(:,:,jk,jl), sye (:,:,jk,jl),   &
                     &                                      syye(:,:,jk,jl), sxye(:,:,jk,jl) )
               END DO
               IF ( nn_pnd_scheme > 0 ) THEN
                  CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0ap  (:,:,jl), sxap (:,:,jl),   &    !--- melt pond fraction --
                     &                                      sxxap (:,:,jl), syap (:,:,jl), syyap (:,:,jl), sxyap (:,:,jl)  )
                  CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0ap  (:,:,jl), sxap (:,:,jl),   & 
                     &                                      sxxap (:,:,jl), syap (:,:,jl), syyap (:,:,jl), sxyap (:,:,jl)  )
                  CALL adv_x( zusnit, pu_ice, 1._wp, zarea, z0vp  (:,:,jl), sxvp (:,:,jl),   &    !--- melt pond volume   --
                     &                                      sxxvp (:,:,jl), syvp (:,:,jl), syyvp (:,:,jl), sxyvp (:,:,jl)  )
                  CALL adv_y( zusnit, pv_ice, 0._wp, zarea, z0vp  (:,:,jl), sxvp (:,:,jl),   & 
                     &                                      sxxvp (:,:,jl), syvp (:,:,jl), syyvp (:,:,jl), sxyvp (:,:,jl)  )
               ENDIF
            END DO
         END DO
      !                                                    !--------------------------------------------!
      ELSE                                                 !== even ice time step:  adv_y then adv_x  ==!
         !                                                 !--------------------------------------------!
         DO jt = 1, initad
            CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0opw (:,:,1), sxopw(:,:),   &             !--- ice open water area
               &                                      sxxopw(:,:)  , syopw(:,:), syyopw(:,:), sxyopw(:,:)  )
            CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0opw (:,:,1), sxopw(:,:),   &
               &                                      sxxopw(:,:)  , syopw(:,:), syyopw(:,:), sxyopw(:,:)  )
            DO jl = 1, jpl
               CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0ice (:,:,jl), sxice(:,:,jl),   &    !--- ice volume  ---
                  &                                      sxxice(:,:,jl), syice(:,:,jl), syyice(:,:,jl), sxyice(:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0ice (:,:,jl), sxice(:,:,jl),   &
                  &                                      sxxice(:,:,jl), syice(:,:,jl), syyice(:,:,jl), sxyice(:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0snw (:,:,jl), sxsn (:,:,jl),   &    !--- snow volume  ---
                  &                                      sxxsn (:,:,jl), sysn (:,:,jl), syysn (:,:,jl), sxysn (:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0snw (:,:,jl), sxsn (:,:,jl),   &
                  &                                      sxxsn (:,:,jl), sysn (:,:,jl), syysn (:,:,jl), sxysn (:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0smi (:,:,jl), sxsal(:,:,jl),   &    !--- ice salinity ---
                  &                                      sxxsal(:,:,jl), sysal(:,:,jl), syysal(:,:,jl), sxysal(:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0smi (:,:,jl), sxsal(:,:,jl),   &
                  &                                      sxxsal(:,:,jl), sysal(:,:,jl), syysal(:,:,jl), sxysal(:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0oi  (:,:,jl), sxage(:,:,jl),   &   !--- ice age      ---
                  &                                      sxxage(:,:,jl), syage(:,:,jl), syyage(:,:,jl), sxyage(:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0oi  (:,:,jl), sxage(:,:,jl),   &
                  &                                      sxxage(:,:,jl), syage(:,:,jl), syyage(:,:,jl), sxyage(:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0ai  (:,:,jl), sxa  (:,:,jl),   &   !--- ice concentrations ---
                  &                                      sxxa  (:,:,jl), sya  (:,:,jl), syya  (:,:,jl), sxya  (:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0ai  (:,:,jl), sxa  (:,:,jl),   &
                  &                                      sxxa  (:,:,jl), sya  (:,:,jl), syya  (:,:,jl), sxya  (:,:,jl)  )
               CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0es  (:,:,jl), sxc0 (:,:,jl),   &  !--- snow heat contents ---
                  &                                      sxxc0 (:,:,jl), syc0 (:,:,jl), syyc0 (:,:,jl), sxyc0 (:,:,jl)  )
               CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0es  (:,:,jl), sxc0 (:,:,jl),   &
                  &                                      sxxc0 (:,:,jl), syc0 (:,:,jl), syyc0 (:,:,jl), sxyc0 (:,:,jl)  )
               DO jk = 1, nlay_i                                                             !--- ice heat contents ---
                  CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0ei(:,:,jk,jl), sxe (:,:,jk,jl),   & 
                     &                                      sxxe(:,:,jk,jl), sye (:,:,jk,jl),   &
                     &                                      syye(:,:,jk,jl), sxye(:,:,jk,jl) )
                  CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0ei(:,:,jk,jl), sxe (:,:,jk,jl),   & 
                     &                                      sxxe(:,:,jk,jl), sye (:,:,jk,jl),   &
                     &                                      syye(:,:,jk,jl), sxye(:,:,jk,jl) )
               END DO
               IF ( nn_pnd_scheme > 0 ) THEN
                  CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0ap  (:,:,jl), sxap (:,:,jl),   &   !--- melt pond fraction ---
                     &                                      sxxap (:,:,jl), syap (:,:,jl), syyap (:,:,jl), sxyap (:,:,jl)  )
                  CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0ap  (:,:,jl), sxap (:,:,jl),   &
                     &                                      sxxap (:,:,jl), syap (:,:,jl), syyap (:,:,jl), sxyap (:,:,jl)  )
                  CALL adv_y( zusnit, pv_ice, 1._wp, zarea, z0vp  (:,:,jl), sxvp (:,:,jl),   &   !--- melt pond volume   ---
                     &                                      sxxvp (:,:,jl), syvp (:,:,jl), syyvp (:,:,jl), sxyvp (:,:,jl)  )
                  CALL adv_x( zusnit, pu_ice, 0._wp, zarea, z0vp  (:,:,jl), sxvp (:,:,jl),   &
                     &                                      sxxvp (:,:,jl), syvp (:,:,jl), syyvp (:,:,jl), sxyvp (:,:,jl)  )
               ENDIF
            END DO
         END DO
      ENDIF

      !-------------------------------------------
      ! Recover the properties from their contents
      !-------------------------------------------
      pato_i(:,:) = z0opw(:,:,1) * r1_e1e2t(:,:)
      DO jl = 1, jpl
         pv_i (:,:,  jl) = z0ice(:,:,jl) * r1_e1e2t(:,:)
         pv_s (:,:,  jl) = z0snw(:,:,jl) * r1_e1e2t(:,:)
         psv_i(:,:,  jl) = z0smi(:,:,jl) * r1_e1e2t(:,:)
         poa_i(:,:,  jl) = z0oi (:,:,jl) * r1_e1e2t(:,:)
         pa_i (:,:,  jl) = z0ai (:,:,jl) * r1_e1e2t(:,:)
         pe_s (:,:,1,jl) = z0es (:,:,jl) * r1_e1e2t(:,:)
         DO jk = 1, nlay_i
            pe_i(:,:,jk,jl) = z0ei(:,:,jk,jl) * r1_e1e2t(:,:)
         END DO
         ! MV MP 2016
         IF ( nn_pnd_scheme > 0 ) THEN
            pa_ip  (:,:,jl)   = z0ap (:,:,jl) * r1_e1e2t(:,:)
            pv_ip  (:,:,jl)   = z0vp (:,:,jl) * r1_e1e2t(:,:)
         ENDIF
         ! END MV MP 2016
      END DO
      !
      DEALLOCATE( zarea , z0opw , z0ice, z0snw , z0ai , z0es , z0smi , z0oi , z0ap , z0vp , z0ei )
      !
      IF( lrst_ice )   CALL adv_pra_rst( 'WRITE', kt )   !* write Prather fields in the restart file
      !
   END SUBROUTINE ice_dyn_adv_pra
   
   SUBROUTINE adv_x( pdf, put , pcrh, psm , ps0 ,   &
      &              psx, psxx, psy , psyy, psxy )
      !!----------------------------------------------------------------------
      !!                **  routine adv_x  **
      !!  
      !! ** purpose :   Computes and adds the advection trend to sea-ice
      !!                variable on x axis
      !!----------------------------------------------------------------------
      REAL(wp)                    , INTENT(in   ) ::   pdf                ! reduction factor for the time step
      REAL(wp)                    , INTENT(in   ) ::   pcrh               ! call adv_x then adv_y (=1) or the opposite (=0)
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   put                ! i-direction ice velocity at U-point [m/s]
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   psm                ! area
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   ps0                ! field to be advected
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   psx , psy          ! 1st moments 
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   psxx, psyy, psxy   ! 2nd moments
      !! 
      INTEGER  ::   ji, jj                               ! dummy loop indices
      REAL(wp) ::   zs1max, zrdt, zslpmax, ztemp         ! local scalars
      REAL(wp) ::   zs1new, zalf , zalfq , zbt           !   -      -
      REAL(wp) ::   zs2new, zalf1, zalf1q, zbt1          !   -      -
      REAL(wp), DIMENSION(jpi,jpj) ::   zf0 , zfx  , zfy   , zbet   ! 2D workspace
      REAL(wp), DIMENSION(jpi,jpj) ::   zfm , zfxx , zfyy  , zfxy   !  -      -
      REAL(wp), DIMENSION(jpi,jpj) ::   zalg, zalg1, zalg1q         !  -      -
      !-----------------------------------------------------------------------

      ! Limitation of moments.                                           

      zrdt = rdt_ice * pdf      ! If ice drift field is too fast, use an appropriate time step for advection.

      DO jj = 1, jpj
         DO ji = 1, jpi
            zslpmax = MAX( 0._wp, ps0(ji,jj) )
            zs1max  = 1.5 * zslpmax
            zs1new  = MIN( zs1max, MAX( -zs1max, psx(ji,jj) ) )
            zs2new  = MIN(  2.0 * zslpmax - 0.3334 * ABS( zs1new ),      &
               &            MAX( ABS( zs1new ) - zslpmax, psxx(ji,jj) )  )
            rswitch = ( 1.0 - MAX( 0._wp, SIGN( 1._wp, -zslpmax) ) ) * tmask(ji,jj,1)   ! Case of empty boxes & Apply mask

            ps0 (ji,jj) = zslpmax  
            psx (ji,jj) = zs1new      * rswitch
            psxx(ji,jj) = zs2new      * rswitch
            psy (ji,jj) = psy (ji,jj) * rswitch
            psyy(ji,jj) = psyy(ji,jj) * rswitch
            psxy(ji,jj) = MIN( zslpmax, MAX( -zslpmax, psxy(ji,jj) ) ) * rswitch
         END DO
      END DO

      !  Initialize volumes of boxes  (=area if adv_x first called, =psm otherwise)                                     
      psm (:,:)  = MAX( pcrh * e1e2t(:,:) + ( 1.0 - pcrh ) * psm(:,:) , epsi20 )

      !  Calculate fluxes and moments between boxes i<-->i+1              
      DO jj = 1, jpj                      !  Flux from i to i+1 WHEN u GT 0 
         DO ji = 1, jpi
            zbet(ji,jj)  =  MAX( 0._wp, SIGN( 1._wp, put(ji,jj) ) )
            zalf         =  MAX( 0._wp, put(ji,jj) ) * zrdt * e2u(ji,jj) / psm(ji,jj)
            zalfq        =  zalf * zalf
            zalf1        =  1.0 - zalf
            zalf1q       =  zalf1 * zalf1
            !
            zfm (ji,jj)  =  zalf  *   psm (ji,jj)
            zf0 (ji,jj)  =  zalf  * ( ps0 (ji,jj) + zalf1 * ( psx(ji,jj) + (zalf1 - zalf) * psxx(ji,jj) )  )
            zfx (ji,jj)  =  zalfq * ( psx (ji,jj) + 3.0 * zalf1 * psxx(ji,jj) )
            zfxx(ji,jj)  =  zalf  *   psxx(ji,jj) * zalfq
            zfy (ji,jj)  =  zalf  * ( psy (ji,jj) + zalf1 * psxy(ji,jj) )
            zfxy(ji,jj)  =  zalfq *   psxy(ji,jj)
            zfyy(ji,jj)  =  zalf  *   psyy(ji,jj)

            !  Readjust moments remaining in the box.
            psm (ji,jj)  =  psm (ji,jj) - zfm(ji,jj)
            ps0 (ji,jj)  =  ps0 (ji,jj) - zf0(ji,jj)
            psx (ji,jj)  =  zalf1q * ( psx(ji,jj) - 3.0 * zalf * psxx(ji,jj) )
            psxx(ji,jj)  =  zalf1  * zalf1q * psxx(ji,jj)
            psy (ji,jj)  =  psy (ji,jj) - zfy(ji,jj)
            psyy(ji,jj)  =  psyy(ji,jj) - zfyy(ji,jj)
            psxy(ji,jj)  =  zalf1q * psxy(ji,jj)
         END DO
      END DO

      DO jj = 1, jpjm1                      !  Flux from i+1 to i when u LT 0.
         DO ji = 1, fs_jpim1
            zalf          = MAX( 0._wp, -put(ji,jj) ) * zrdt * e2u(ji,jj) / psm(ji+1,jj) 
            zalg  (ji,jj) = zalf
            zalfq         = zalf * zalf
            zalf1         = 1.0 - zalf
            zalg1 (ji,jj) = zalf1
            zalf1q        = zalf1 * zalf1
            zalg1q(ji,jj) = zalf1q
            !
            zfm   (ji,jj) = zfm (ji,jj) + zalf  *   psm (ji+1,jj)
            zf0   (ji,jj) = zf0 (ji,jj) + zalf  * ( ps0 (ji+1,jj) - zalf1 * ( psx(ji+1,jj) - (zalf1 - zalf ) * psxx(ji+1,jj) ) )
            zfx   (ji,jj) = zfx (ji,jj) + zalfq * ( psx (ji+1,jj) - 3.0 * zalf1 * psxx(ji+1,jj) )
            zfxx  (ji,jj) = zfxx(ji,jj) + zalf  *   psxx(ji+1,jj) * zalfq
            zfy   (ji,jj) = zfy (ji,jj) + zalf  * ( psy (ji+1,jj) - zalf1 * psxy(ji+1,jj) )
            zfxy  (ji,jj) = zfxy(ji,jj) + zalfq *   psxy(ji+1,jj)
            zfyy  (ji,jj) = zfyy(ji,jj) + zalf  *   psyy(ji+1,jj)
         END DO
      END DO

      DO jj = 2, jpjm1                     !  Readjust moments remaining in the box. 
         DO ji = fs_2, fs_jpim1
            zbt  =       zbet(ji-1,jj)
            zbt1 = 1.0 - zbet(ji-1,jj)
            !
            psm (ji,jj) = zbt * psm(ji,jj) + zbt1 * ( psm(ji,jj) - zfm(ji-1,jj) )
            ps0 (ji,jj) = zbt * ps0(ji,jj) + zbt1 * ( ps0(ji,jj) - zf0(ji-1,jj) )
            psx (ji,jj) = zalg1q(ji-1,jj) * ( psx(ji,jj) + 3.0 * zalg(ji-1,jj) * psxx(ji,jj) )
            psxx(ji,jj) = zalg1 (ji-1,jj) * zalg1q(ji-1,jj) * psxx(ji,jj)
            psy (ji,jj) = zbt * psy (ji,jj) + zbt1 * ( psy (ji,jj) - zfy (ji-1,jj) )
            psyy(ji,jj) = zbt * psyy(ji,jj) + zbt1 * ( psyy(ji,jj) - zfyy(ji-1,jj) )
            psxy(ji,jj) = zalg1q(ji-1,jj) * psxy(ji,jj)
         END DO
      END DO

      !   Put the temporary moments into appropriate neighboring boxes.    
      DO jj = 2, jpjm1                     !   Flux from i to i+1 IF u GT 0.
         DO ji = fs_2, fs_jpim1
            zbt  =       zbet(ji-1,jj)
            zbt1 = 1.0 - zbet(ji-1,jj)
            psm(ji,jj)  = zbt * ( psm(ji,jj) + zfm(ji-1,jj) ) + zbt1 * psm(ji,jj)
            zalf        = zbt * zfm(ji-1,jj) / psm(ji,jj)
            zalf1       = 1.0 - zalf
            ztemp       = zalf * ps0(ji,jj) - zalf1 * zf0(ji-1,jj)
            !
            ps0 (ji,jj) = zbt * ( ps0(ji,jj) + zf0(ji-1,jj) ) + zbt1 * ps0(ji,jj)
            psx (ji,jj) = zbt * ( zalf * zfx(ji-1,jj) + zalf1 * psx(ji,jj) + 3.0 * ztemp ) + zbt1 * psx(ji,jj)
            psxx(ji,jj) = zbt * ( zalf * zalf * zfxx(ji-1,jj) + zalf1 * zalf1 * psxx(ji,jj)                               &
               &                + 5.0 * ( zalf * zalf1 * ( psx (ji,jj) - zfx(ji-1,jj) ) - ( zalf1 - zalf ) * ztemp )  )   &
               &                                                + zbt1 * psxx(ji,jj)
            psxy(ji,jj) = zbt * ( zalf * zfxy(ji-1,jj) + zalf1 * psxy(ji,jj)             &
               &                + 3.0 * (- zalf1*zfy(ji-1,jj)  + zalf * psy(ji,jj) ) )   &
               &                                                + zbt1 * psxy(ji,jj)
            psy (ji,jj) = zbt * ( psy (ji,jj) + zfy (ji-1,jj) ) + zbt1 * psy (ji,jj)
            psyy(ji,jj) = zbt * ( psyy(ji,jj) + zfyy(ji-1,jj) ) + zbt1 * psyy(ji,jj)
         END DO
      END DO

      DO jj = 2, jpjm1                     !  Flux from i+1 to i IF u LT 0.
         DO ji = fs_2, fs_jpim1
            zbt  =       zbet(ji,jj)
            zbt1 = 1.0 - zbet(ji,jj)
            psm(ji,jj)  = zbt * psm(ji,jj)  + zbt1 * ( psm(ji,jj) + zfm(ji,jj) )
            zalf        = zbt1 * zfm(ji,jj) / psm(ji,jj)
            zalf1       = 1.0 - zalf
            ztemp       = - zalf * ps0(ji,jj) + zalf1 * zf0(ji,jj)
            !
            ps0(ji,jj)  = zbt * ps0 (ji,jj) + zbt1 * ( ps0(ji,jj) + zf0(ji,jj) )
            psx(ji,jj)  = zbt * psx (ji,jj) + zbt1 * ( zalf * zfx(ji,jj) + zalf1 * psx(ji,jj) + 3.0 * ztemp )
            psxx(ji,jj) = zbt * psxx(ji,jj) + zbt1 * ( zalf * zalf * zfxx(ji,jj)  + zalf1 * zalf1 * psxx(ji,jj)  &
               &                                      + 5.0 *( zalf * zalf1 * ( - psx(ji,jj) + zfx(ji,jj) )      &
               &                                      + ( zalf1 - zalf ) * ztemp ) )
            psxy(ji,jj) = zbt * psxy(ji,jj) + zbt1 * (  zalf * zfxy(ji,jj) + zalf1 * psxy(ji,jj)  &
               &                                      + 3.0 * ( zalf1 * zfy(ji,jj) - zalf * psy(ji,jj) )  )
            psy(ji,jj)  = zbt * psy (ji,jj)  + zbt1 * ( psy (ji,jj) + zfy (ji,jj) )
            psyy(ji,jj) = zbt * psyy(ji,jj)  + zbt1 * ( psyy(ji,jj) + zfyy(ji,jj) )
         END DO
      END DO

      !-- Lateral boundary conditions
      CALL lbc_lnk_multi( psm , 'T',  1., ps0 , 'T',  1.   &
         &              , psx , 'T', -1., psy , 'T', -1.   &   ! caution gradient ==> the sign changes
         &              , psxx, 'T',  1., psyy, 'T',  1.   &
         &              , psxy, 'T',  1. )

      IF(ln_ctl) THEN
         CALL prt_ctl(tab2d_1=psm  , clinfo1=' adv_x: psm  :', tab2d_2=ps0 , clinfo2=' ps0  : ')
         CALL prt_ctl(tab2d_1=psx  , clinfo1=' adv_x: psx  :', tab2d_2=psxx, clinfo2=' psxx : ')
         CALL prt_ctl(tab2d_1=psy  , clinfo1=' adv_x: psy  :', tab2d_2=psyy, clinfo2=' psyy : ')
         CALL prt_ctl(tab2d_1=psxy , clinfo1=' adv_x: psxy :')
      ENDIF
      !
   END SUBROUTINE adv_x


   SUBROUTINE adv_y( pdf, pvt , pcrh, psm , ps0 ,   &
      &              psx, psxx, psy , psyy, psxy )
      !!---------------------------------------------------------------------
      !!                **  routine adv_y  **
      !!            
      !! ** purpose :   Computes and adds the advection trend to sea-ice 
      !!                variable on y axis
      !!---------------------------------------------------------------------
      REAL(wp)                    , INTENT(in   ) ::   pdf                ! reduction factor for the time step
      REAL(wp)                    , INTENT(in   ) ::   pcrh               ! call adv_x then adv_y (=1) or the opposite (=0)
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   pvt                ! j-direction ice velocity at V-point [m/s]
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   psm                ! area
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   ps0                ! field to be advected
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   psx , psy          ! 1st moments 
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout) ::   psxx, psyy, psxy   ! 2nd moments
      !!
      INTEGER  ::   ji, jj                               ! dummy loop indices
      REAL(wp) ::   zs1max, zrdt, zslpmax, ztemp         ! temporary scalars
      REAL(wp) ::   zs1new, zalf , zalfq , zbt           !    -         -
      REAL(wp) ::   zs2new, zalf1, zalf1q, zbt1          !    -         -
      REAL(wp), DIMENSION(jpi,jpj) ::   zf0, zfx , zfy , zbet   ! 2D workspace
      REAL(wp), DIMENSION(jpi,jpj) ::   zfm, zfxx, zfyy, zfxy   !  -      -
      REAL(wp), DIMENSION(jpi,jpj) ::   zalg, zalg1, zalg1q     !  -      -
      !---------------------------------------------------------------------

      ! Limitation of moments.

      zrdt = rdt_ice * pdf ! If ice drift field is too fast, use an appropriate time step for advection.

      DO jj = 1, jpj
         DO ji = 1, jpi
            zslpmax = MAX( 0._wp, ps0(ji,jj) )
            zs1max  = 1.5 * zslpmax
            zs1new  = MIN( zs1max, MAX( -zs1max, psy(ji,jj) ) )
            zs2new  = MIN(  ( 2.0 * zslpmax - 0.3334 * ABS( zs1new ) ),   &
               &             MAX( ABS( zs1new )-zslpmax, psyy(ji,jj) )  )
            rswitch = ( 1.0 - MAX( 0._wp, SIGN( 1._wp, -zslpmax) ) ) * tmask(ji,jj,1)   ! Case of empty boxes & Apply mask
            !
            ps0 (ji,jj) = zslpmax  
            psx (ji,jj) = psx (ji,jj) * rswitch
            psxx(ji,jj) = psxx(ji,jj) * rswitch
            psy (ji,jj) = zs1new * rswitch
            psyy(ji,jj) = zs2new * rswitch
            psxy(ji,jj) = MIN( zslpmax, MAX( -zslpmax, psxy(ji,jj) ) ) * rswitch
         END DO
      END DO

      !  Initialize volumes of boxes (=area if adv_x first called, =psm otherwise)
      psm(:,:)  = MAX(  pcrh * e1e2t(:,:) + ( 1.0 - pcrh ) * psm(:,:) , epsi20  )

      !  Calculate fluxes and moments between boxes j<-->j+1              
      DO jj = 1, jpj                     !  Flux from j to j+1 WHEN v GT 0   
         DO ji = 1, jpi
            zbet(ji,jj)  =  MAX( 0._wp, SIGN( 1._wp, pvt(ji,jj) ) )
            zalf         =  MAX( 0._wp, pvt(ji,jj) ) * zrdt * e1v(ji,jj) / psm(ji,jj)
            zalfq        =  zalf * zalf
            zalf1        =  1.0 - zalf
            zalf1q       =  zalf1 * zalf1
            !
            zfm (ji,jj)  =  zalf  * psm(ji,jj)
            zf0 (ji,jj)  =  zalf  * ( ps0(ji,jj) + zalf1 * ( psy(ji,jj)  + (zalf1-zalf) * psyy(ji,jj)  ) ) 
            zfy (ji,jj)  =  zalfq *( psy(ji,jj) + 3.0*zalf1*psyy(ji,jj) )
            zfyy(ji,jj)  =  zalf  * zalfq * psyy(ji,jj)
            zfx (ji,jj)  =  zalf  * ( psx(ji,jj) + zalf1 * psxy(ji,jj) )
            zfxy(ji,jj)  =  zalfq * psxy(ji,jj)
            zfxx(ji,jj)  =  zalf  * psxx(ji,jj)
            !
            !  Readjust moments remaining in the box.
            psm (ji,jj)  =  psm (ji,jj) - zfm(ji,jj)
            ps0 (ji,jj)  =  ps0 (ji,jj) - zf0(ji,jj)
            psy (ji,jj)  =  zalf1q * ( psy(ji,jj) -3.0 * zalf * psyy(ji,jj) )
            psyy(ji,jj)  =  zalf1 * zalf1q * psyy(ji,jj)
            psx (ji,jj)  =  psx (ji,jj) - zfx(ji,jj)
            psxx(ji,jj)  =  psxx(ji,jj) - zfxx(ji,jj)
            psxy(ji,jj)  =  zalf1q * psxy(ji,jj)
         END DO
      END DO
      !
      DO jj = 1, jpjm1                   !  Flux from j+1 to j when v LT 0.
         DO ji = 1, jpi
            zalf          = ( MAX(0._wp, -pvt(ji,jj) ) * zrdt * e1v(ji,jj) ) / psm(ji,jj+1) 
            zalg  (ji,jj) = zalf
            zalfq         = zalf * zalf
            zalf1         = 1.0 - zalf
            zalg1 (ji,jj) = zalf1
            zalf1q        = zalf1 * zalf1
            zalg1q(ji,jj) = zalf1q
            !
            zfm   (ji,jj) = zfm (ji,jj) + zalf  *   psm (ji,jj+1)
            zf0   (ji,jj) = zf0 (ji,jj) + zalf  * ( ps0 (ji,jj+1) - zalf1 * (psy(ji,jj+1) - (zalf1 - zalf ) * psyy(ji,jj+1) ) )
            zfy   (ji,jj) = zfy (ji,jj) + zalfq * ( psy (ji,jj+1) - 3.0 * zalf1 * psyy(ji,jj+1) )
            zfyy  (ji,jj) = zfyy(ji,jj) + zalf  *   psyy(ji,jj+1) * zalfq
            zfx   (ji,jj) = zfx (ji,jj) + zalf  * ( psx (ji,jj+1) - zalf1 * psxy(ji,jj+1) )
            zfxy  (ji,jj) = zfxy(ji,jj) + zalfq *   psxy(ji,jj+1)
            zfxx  (ji,jj) = zfxx(ji,jj) + zalf  *   psxx(ji,jj+1)
         END DO
      END DO

      !  Readjust moments remaining in the box. 
      DO jj = 2, jpj
         DO ji = 1, jpi
            zbt  =         zbet(ji,jj-1)
            zbt1 = ( 1.0 - zbet(ji,jj-1) )
            !
            psm (ji,jj) = zbt * psm(ji,jj) + zbt1 * ( psm(ji,jj) - zfm(ji,jj-1) )
            ps0 (ji,jj) = zbt * ps0(ji,jj) + zbt1 * ( ps0(ji,jj) - zf0(ji,jj-1) )
            psy (ji,jj) = zalg1q(ji,jj-1) * ( psy(ji,jj) + 3.0 * zalg(ji,jj-1) * psyy(ji,jj) )
            psyy(ji,jj) = zalg1 (ji,jj-1) * zalg1q(ji,jj-1) * psyy(ji,jj)
            psx (ji,jj) = zbt * psx (ji,jj) + zbt1 * ( psx (ji,jj) - zfx (ji,jj-1) )
            psxx(ji,jj) = zbt * psxx(ji,jj) + zbt1 * ( psxx(ji,jj) - zfxx(ji,jj-1) )
            psxy(ji,jj) = zalg1q(ji,jj-1) * psxy(ji,jj)
         END DO
      END DO

      !   Put the temporary moments into appropriate neighboring boxes.    
      DO jj = 2, jpjm1                    !   Flux from j to j+1 IF v GT 0.
         DO ji = 1, jpi
            zbt  =         zbet(ji,jj-1)
            zbt1 = ( 1.0 - zbet(ji,jj-1) )
            psm(ji,jj)  = zbt * ( psm(ji,jj) + zfm(ji,jj-1) ) + zbt1 * psm(ji,jj) 
            zalf        = zbt * zfm(ji,jj-1) / psm(ji,jj) 
            zalf1       = 1.0 - zalf
            ztemp       = zalf * ps0(ji,jj) - zalf1 * zf0(ji,jj-1)
            !
            ps0(ji,jj)  = zbt  * ( ps0(ji,jj) + zf0(ji,jj-1) ) + zbt1 * ps0(ji,jj)
            psy(ji,jj)  = zbt  * ( zalf * zfy(ji,jj-1) + zalf1 * psy(ji,jj) + 3.0 * ztemp )   &
               &                                               + zbt1 * psy(ji,jj)  
            psyy(ji,jj) = zbt  * ( zalf * zalf * zfyy(ji,jj-1) + zalf1 * zalf1 * psyy(ji,jj)                             &
               &                 + 5.0 * ( zalf * zalf1 * ( psy(ji,jj) - zfy(ji,jj-1) ) - ( zalf1 - zalf ) * ztemp ) )   & 
               &                                               + zbt1 * psyy(ji,jj)
            psxy(ji,jj) = zbt  * (  zalf * zfxy(ji,jj-1) + zalf1 * psxy(ji,jj)               &
               &                  + 3.0 * (- zalf1 * zfx(ji,jj-1) + zalf * psx(ji,jj) )  )   &
               &                                                + zbt1 * psxy(ji,jj)
            psx (ji,jj) = zbt * ( psx (ji,jj) + zfx (ji,jj-1) ) + zbt1 * psx (ji,jj)
            psxx(ji,jj) = zbt * ( psxx(ji,jj) + zfxx(ji,jj-1) ) + zbt1 * psxx(ji,jj)
         END DO
      END DO

      DO jj = 2, jpjm1                   !  Flux from j+1 to j IF v LT 0.
         DO ji = 1, jpi
            zbt  =         zbet(ji,jj)
            zbt1 = ( 1.0 - zbet(ji,jj) )
            psm(ji,jj)  = zbt * psm(ji,jj) + zbt1 * ( psm(ji,jj) + zfm(ji,jj) )
            zalf        = zbt1 * zfm(ji,jj) / psm(ji,jj)
            zalf1       = 1.0 - zalf
            ztemp       = - zalf * ps0 (ji,jj) + zalf1 * zf0(ji,jj)
            ps0 (ji,jj) =   zbt  * ps0 (ji,jj) + zbt1  * ( ps0(ji,jj) + zf0(ji,jj) )
            psy (ji,jj) =   zbt  * psy (ji,jj) + zbt1  * ( zalf * zfy(ji,jj) + zalf1 * psy(ji,jj) + 3.0 * ztemp )
            psyy(ji,jj) =   zbt  * psyy(ji,jj) + zbt1 * (  zalf * zalf * zfyy(ji,jj) + zalf1 * zalf1 * psyy(ji,jj)   &
               &                                         + 5.0 *( zalf *zalf1 *( -psy(ji,jj) + zfy(ji,jj) )          &
               &                                         + ( zalf1 - zalf ) * ztemp )                                )
            psxy(ji,jj) =   zbt  * psxy(ji,jj) + zbt1 * (  zalf * zfxy(ji,jj) + zalf1 * psxy(ji,jj)       &
               &                                         + 3.0 * ( zalf1 * zfx(ji,jj) - zalf * psx(ji,jj) )  )
            psx (ji,jj) =   zbt  * psx (ji,jj) + zbt1 * ( psx (ji,jj) + zfx (ji,jj) )
            psxx(ji,jj) =   zbt  * psxx(ji,jj) + zbt1 * ( psxx(ji,jj) + zfxx(ji,jj) )
         END DO
      END DO

      !-- Lateral boundary conditions
      CALL lbc_lnk_multi( psm , 'T',  1.,  ps0 , 'T',  1.   &
         &              , psx , 'T', -1.,  psy , 'T', -1.   &   ! caution gradient ==> the sign changes
         &              , psxx, 'T',  1.,  psyy, 'T',  1.   &
         &              , psxy, 'T',  1. )

      IF(ln_ctl) THEN
         CALL prt_ctl(tab2d_1=psm  , clinfo1=' adv_y: psm  :', tab2d_2=ps0 , clinfo2=' ps0  : ')
         CALL prt_ctl(tab2d_1=psx  , clinfo1=' adv_y: psx  :', tab2d_2=psxx, clinfo2=' psxx : ')
         CALL prt_ctl(tab2d_1=psy  , clinfo1=' adv_y: psy  :', tab2d_2=psyy, clinfo2=' psyy : ')
         CALL prt_ctl(tab2d_1=psxy , clinfo1=' adv_y: psxy :')
      ENDIF
      !
   END SUBROUTINE adv_y

   SUBROUTINE adv_pra_init
      !!-------------------------------------------------------------------
      !!                  ***  ROUTINE adv_pra_init  ***
      !!
      !! ** Purpose :   allocate and initialize arrays for Prather advection 
      !!-------------------------------------------------------------------
      INTEGER ::   ierr
      !!-------------------------------------------------------------------
      ALLOCATE( sxopw(jpi,jpj)     , syopw(jpi,jpj)     , sxxopw(jpi,jpj)     , syyopw(jpi,jpj)     , sxyopw(jpi,jpj)     ,   &
         &      sxice(jpi,jpj,jpl) , syice(jpi,jpj,jpl) , sxxice(jpi,jpj,jpl) , syyice(jpi,jpj,jpl) , sxyice(jpi,jpj,jpl) ,   &
         &      sxsn (jpi,jpj,jpl) , sysn (jpi,jpj,jpl) , sxxsn (jpi,jpj,jpl) , syysn (jpi,jpj,jpl) , sxysn (jpi,jpj,jpl) ,   &
         &      sxa  (jpi,jpj,jpl) , sya  (jpi,jpj,jpl) , sxxa  (jpi,jpj,jpl) , syya  (jpi,jpj,jpl) , sxya  (jpi,jpj,jpl) ,   &
         &      sxc0 (jpi,jpj,jpl) , syc0 (jpi,jpj,jpl) , sxxc0 (jpi,jpj,jpl) , syyc0 (jpi,jpj,jpl) , sxyc0 (jpi,jpj,jpl) ,   &
         &      sxsal(jpi,jpj,jpl) , sysal(jpi,jpj,jpl) , sxxsal(jpi,jpj,jpl) , syysal(jpi,jpj,jpl) , sxysal(jpi,jpj,jpl) ,   &
         &      sxage(jpi,jpj,jpl) , syage(jpi,jpj,jpl) , sxxage(jpi,jpj,jpl) , syyage(jpi,jpj,jpl) , sxyage(jpi,jpj,jpl) ,   &
         &      sxap(jpi,jpj,jpl)  , syap (jpi,jpj,jpl) , sxxap (jpi,jpj,jpl) , syyap (jpi,jpj,jpl) , sxyap (jpi,jpj,jpl) ,   &
         &      sxvp(jpi,jpj,jpl)  , syvp (jpi,jpj,jpl) , sxxvp (jpi,jpj,jpl) , syyvp (jpi,jpj,jpl) , sxyvp (jpi,jpj,jpl) ,   &
         &      sxe (jpi,jpj,nlay_i,jpl) , sye (jpi,jpj,nlay_i,jpl) , sxxe(jpi,jpj,nlay_i,jpl) , &
         &      syye(jpi,jpj,nlay_i,jpl) , sxye(jpi,jpj,nlay_i,jpl)                            , &
         &      STAT = ierr )
      !
      IF( lk_mpp    )   CALL mpp_sum( ierr )
      IF( ierr /= 0 )   CALL ctl_stop('STOP', 'adv_pra_init : unable to allocate ice arrays for Prather advection scheme')
      !
      CALL adv_pra_rst( 'READ' )  !* read or initialize all required files
      !
   END SUBROUTINE adv_pra_init

   SUBROUTINE adv_pra_rst( cdrw, kt )
      !!---------------------------------------------------------------------
      !!                   ***  ROUTINE adv_pra_rst  ***
      !!                     
      !! ** Purpose :   Read or write RHG file in restart file
      !!
      !! ** Method  :   use of IOM library
      !!----------------------------------------------------------------------
      CHARACTER(len=*) , INTENT(in) ::   cdrw   ! "READ"/"WRITE" flag
      INTEGER, OPTIONAL, INTENT(in) ::   kt     ! ice time-step
      !
      INTEGER ::   jk, jl   ! dummy loop indices
      INTEGER ::   iter     ! local integer
      INTEGER ::   id1      ! local integer
      CHARACTER(len=25) ::   znam
      CHARACTER(len=2)  ::   zchar, zchar1
      REAL(wp), DIMENSION(jpi,jpj) :: z2d
      !!----------------------------------------------------------------------
      !
      IF( TRIM(cdrw) == 'READ' ) THEN        ! Read/initialize
         !                                   ! ---------------
         IF( ln_rstart ) THEN                   !* Read the restart file
            !
            id1 = iom_varid( numrir, 'sxopw' , ldstop = .FALSE. )
            !
            IF( id1 > 0 ) THEN      ! fields exist
               DO jl = 1, jpl 
                  WRITE(zchar,'(I2.2)') jl
                  znam = 'sxice'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxice(:,:,jl) = z2d(:,:)
                  znam = 'syice'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syice(:,:,jl) = z2d(:,:)
                  znam = 'sxxice'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxxice(:,:,jl) = z2d(:,:)
                  znam = 'syyice'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syyice(:,:,jl) = z2d(:,:)
                  znam = 'sxyice'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxyice(:,:,jl) = z2d(:,:)
                  znam = 'sxsn'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxsn(:,:,jl) = z2d(:,:)
                  znam = 'sysn'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sysn(:,:,jl) = z2d(:,:)
                  znam = 'sxxsn'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxxsn(:,:,jl) = z2d(:,:)
                  znam = 'syysn'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syysn(:,:,jl) = z2d(:,:)
                  znam = 'sxysn'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxysn(:,:,jl) = z2d(:,:)
                  znam = 'sxa'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxa(:,:,jl) = z2d(:,:)
                  znam = 'sya'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sya(:,:,jl) = z2d(:,:)
                  znam = 'sxxa'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxxa(:,:,jl) = z2d(:,:)
                  znam = 'syya'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syya(:,:,jl) = z2d(:,:)
                  znam = 'sxya'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxya(:,:,jl) = z2d(:,:)
                  znam = 'sxc0'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxc0(:,:,jl) = z2d(:,:)
                  znam = 'syc0'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syc0(:,:,jl) = z2d(:,:)
                  znam = 'sxxc0'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxxc0(:,:,jl) = z2d(:,:)
                  znam = 'syyc0'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syyc0(:,:,jl) = z2d(:,:)
                  znam = 'sxyc0'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxyc0(:,:,jl) = z2d(:,:)
                  znam = 'sxsal'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxsal(:,:,jl) = z2d(:,:)
                  znam = 'sysal'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sysal(:,:,jl) = z2d(:,:)
                  znam = 'sxxsal'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxxsal(:,:,jl) = z2d(:,:)
                  znam = 'syysal'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syysal(:,:,jl) = z2d(:,:)
                  znam = 'sxysal'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxysal(:,:,jl) = z2d(:,:)
                  znam = 'sxage'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxage(:,:,jl) = z2d(:,:)
                  znam = 'syage'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syage(:,:,jl) = z2d(:,:)
                  znam = 'sxxage'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxxage(:,:,jl) = z2d(:,:)
                  znam = 'syyage'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  syyage(:,:,jl) = z2d(:,:)
                  znam = 'sxyage'//'_htc'//zchar
                  CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                  sxyage(:,:,jl)= z2d(:,:)
               END DO
               IF ( nn_pnd_scheme > 0 ) THEN
                  DO jl = 1, jpl 
                     WRITE(zchar,'(I2.2)') jl
                     znam = 'sxap'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxap(:,:,jl) = z2d(:,:)
                     znam = 'syap'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     syap(:,:,jl) = z2d(:,:)
                     znam = 'sxxap'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxxap(:,:,jl) = z2d(:,:)
                     znam = 'syyap'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     syyap(:,:,jl) = z2d(:,:)
                     znam = 'sxyap'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxyap(:,:,jl) = z2d(:,:)

                     znam = 'sxvp'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxvp(:,:,jl) = z2d(:,:)
                     znam = 'syvp'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     syvp(:,:,jl) = z2d(:,:)
                     znam = 'sxxvp'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxxvp(:,:,jl) = z2d(:,:)
                     znam = 'syyvp'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     syyvp(:,:,jl) = z2d(:,:)
                     znam = 'sxyvp'//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxyvp(:,:,jl) = z2d(:,:)
                  END DO
               ENDIF

               CALL iom_get( numrir, jpdom_autoglo, 'sxopw ' ,  sxopw  )
               CALL iom_get( numrir, jpdom_autoglo, 'syopw ' ,  syopw  )
               CALL iom_get( numrir, jpdom_autoglo, 'sxxopw' ,  sxxopw )
               CALL iom_get( numrir, jpdom_autoglo, 'syyopw' ,  syyopw )
               CALL iom_get( numrir, jpdom_autoglo, 'sxyopw' ,  sxyopw )

               DO jl = 1, jpl 
                  WRITE(zchar,'(I2.2)') jl
                  DO jk = 1, nlay_i 
                     WRITE(zchar1,'(I2.2)') jk
                     znam = 'sxe'//'_il'//zchar1//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxe(:,:,jk,jl) = z2d(:,:)
                     znam = 'sye'//'_il'//zchar1//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sye(:,:,jk,jl) = z2d(:,:)
                     znam = 'sxxe'//'_il'//zchar1//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxxe(:,:,jk,jl) = z2d(:,:)
                     znam = 'syye'//'_il'//zchar1//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     syye(:,:,jk,jl) = z2d(:,:)
                     znam = 'sxye'//'_il'//zchar1//'_htc'//zchar
                     CALL iom_get( numrir, jpdom_autoglo, znam , z2d )
                     sxye(:,:,jk,jl) = z2d(:,:)
                  END DO
               END DO
               !
            ELSE                                     ! start rheology from rest
               IF(lwp) WRITE(numout,*) '   ==>>   previous run without Prather, set moments to 0'
               sxopw (:,:) = 0._wp   ;   sxice (:,:,:) = 0._wp   ;   sxsn (:,:,:)  = 0._wp   ;   sxa  (:,:,:) = 0._wp
               syopw (:,:) = 0._wp   ;   syice (:,:,:) = 0._wp   ;   sysn (:,:,:)  = 0._wp   ;   sya  (:,:,:) = 0._wp
               sxxopw(:,:) = 0._wp   ;   sxxice(:,:,:) = 0._wp   ;   sxxsn(:,:,:)  = 0._wp   ;   sxxa (:,:,:) = 0._wp
               syyopw(:,:) = 0._wp   ;   syyice(:,:,:) = 0._wp   ;   syysn(:,:,:)  = 0._wp   ;   syya (:,:,:) = 0._wp
               sxyopw(:,:) = 0._wp   ;   sxyice(:,:,:) = 0._wp   ;   sxysn(:,:,:)  = 0._wp   ;   sxya (:,:,:) = 0._wp
               !
               sxc0  (:,:,:) = 0._wp   ;   sxe  (:,:,:,:) = 0._wp   ;   sxsal  (:,:,:) = 0._wp   ;   sxage  (:,:,:) = 0._wp
               syc0  (:,:,:) = 0._wp   ;   sye  (:,:,:,:) = 0._wp   ;   sysal  (:,:,:) = 0._wp   ;   syage  (:,:,:) = 0._wp
               sxxc0 (:,:,:) = 0._wp   ;   sxxe (:,:,:,:) = 0._wp   ;   sxxsal (:,:,:) = 0._wp   ;   sxxage (:,:,:) = 0._wp
               syyc0 (:,:,:) = 0._wp   ;   syye (:,:,:,:) = 0._wp   ;   syysal (:,:,:) = 0._wp   ;   syyage (:,:,:) = 0._wp
               sxyc0 (:,:,:) = 0._wp   ;   sxye (:,:,:,:) = 0._wp   ;   sxysal (:,:,:) = 0._wp   ;   sxyage (:,:,:) = 0._wp
               IF ( nn_pnd_scheme > 0 ) THEN
                  sxap  (:,:,:) = 0._wp    ; sxvp  (:,:,:) = 0._wp 
                  syap  (:,:,:) = 0._wp    ; syvp  (:,:,:) = 0._wp 
                  sxxap (:,:,:) = 0._wp    ; sxxvp (:,:,:) = 0._wp 
                  syyap (:,:,:) = 0._wp    ; syyvp (:,:,:) = 0._wp 
                  sxyap (:,:,:) = 0._wp    ; sxyvp (:,:,:) = 0._wp
               ENDIF
            ENDIF
         ELSE                                   !* Start from rest
            IF(lwp) WRITE(numout,*) '   ==>>   start from rest: set moments to 0'
            sxopw (:,:) = 0._wp   ;   sxice (:,:,:) = 0._wp   ;   sxsn (:,:,:)  = 0._wp   ;   sxa  (:,:,:) = 0._wp
            syopw (:,:) = 0._wp   ;   syice (:,:,:) = 0._wp   ;   sysn (:,:,:)  = 0._wp   ;   sya  (:,:,:) = 0._wp
            sxxopw(:,:) = 0._wp   ;   sxxice(:,:,:) = 0._wp   ;   sxxsn(:,:,:)  = 0._wp   ;   sxxa (:,:,:) = 0._wp
            syyopw(:,:) = 0._wp   ;   syyice(:,:,:) = 0._wp   ;   syysn(:,:,:)  = 0._wp   ;   syya (:,:,:) = 0._wp
            sxyopw(:,:) = 0._wp   ;   sxyice(:,:,:) = 0._wp   ;   sxysn(:,:,:)  = 0._wp   ;   sxya (:,:,:) = 0._wp
            !
            sxc0  (:,:,:) = 0._wp   ;   sxe  (:,:,:,:) = 0._wp   ;   sxsal  (:,:,:) = 0._wp   ;   sxage  (:,:,:) = 0._wp
            syc0  (:,:,:) = 0._wp   ;   sye  (:,:,:,:) = 0._wp   ;   sysal  (:,:,:) = 0._wp   ;   syage  (:,:,:) = 0._wp
            sxxc0 (:,:,:) = 0._wp   ;   sxxe (:,:,:,:) = 0._wp   ;   sxxsal (:,:,:) = 0._wp   ;   sxxage (:,:,:) = 0._wp
            syyc0 (:,:,:) = 0._wp   ;   syye (:,:,:,:) = 0._wp   ;   syysal (:,:,:) = 0._wp   ;   syyage (:,:,:) = 0._wp
            sxyc0 (:,:,:) = 0._wp   ;   sxye (:,:,:,:) = 0._wp   ;   sxysal (:,:,:) = 0._wp   ;   sxyage (:,:,:) = 0._wp
            IF ( nn_pnd_scheme > 0 ) THEN
               sxap  (:,:,:) = 0._wp    ; sxvp  (:,:,:) = 0._wp 
               syap  (:,:,:) = 0._wp    ; syvp  (:,:,:) = 0._wp 
               sxxap (:,:,:) = 0._wp    ; sxxvp (:,:,:) = 0._wp 
               syyap (:,:,:) = 0._wp    ; syyvp (:,:,:) = 0._wp 
               sxyap (:,:,:) = 0._wp    ; sxyvp (:,:,:) = 0._wp
            ENDIF
         ENDIF
         !
      ELSEIF( TRIM(cdrw) == 'WRITE' ) THEN   ! Create restart file
         !                                   ! -------------------
         IF(lwp) WRITE(numout,*) '---- adv-rst ----'
         iter = kt + nn_fsbc - 1             ! ice restarts are written at kt == nitrst - nn_fsbc + 1
         !
         DO jl = 1, jpl 
            WRITE(zchar,'(I2.2)') jl
            znam = 'sxice'//'_htc'//zchar
            z2d(:,:) = sxice(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syice'//'_htc'//zchar
            z2d(:,:) = syice(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxxice'//'_htc'//zchar
            z2d(:,:) = sxxice(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syyice'//'_htc'//zchar
            z2d(:,:) = syyice(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxyice'//'_htc'//zchar
            z2d(:,:) = sxyice(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxsn'//'_htc'//zchar
            z2d(:,:) = sxsn(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sysn'//'_htc'//zchar
            z2d(:,:) = sysn(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxxsn'//'_htc'//zchar
            z2d(:,:) = sxxsn(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syysn'//'_htc'//zchar
            z2d(:,:) = syysn(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxysn'//'_htc'//zchar
            z2d(:,:) = sxysn(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxa'//'_htc'//zchar
            z2d(:,:) = sxa(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sya'//'_htc'//zchar
            z2d(:,:) = sya(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxxa'//'_htc'//zchar
            z2d(:,:) = sxxa(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syya'//'_htc'//zchar
            z2d(:,:) = syya(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxya'//'_htc'//zchar
            z2d(:,:) = sxya(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxc0'//'_htc'//zchar
            z2d(:,:) = sxc0(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syc0'//'_htc'//zchar
            z2d(:,:) = syc0(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxxc0'//'_htc'//zchar
            z2d(:,:) = sxxc0(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syyc0'//'_htc'//zchar
            z2d(:,:) = syyc0(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxyc0'//'_htc'//zchar
            z2d(:,:) = sxyc0(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxsal'//'_htc'//zchar
            z2d(:,:) = sxsal(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sysal'//'_htc'//zchar
            z2d(:,:) = sysal(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxxsal'//'_htc'//zchar
            z2d(:,:) = sxxsal(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syysal'//'_htc'//zchar
            z2d(:,:) = syysal(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxysal'//'_htc'//zchar
            z2d(:,:) = sxysal(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxage'//'_htc'//zchar
            z2d(:,:) = sxage(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syage'//'_htc'//zchar
            z2d(:,:) = syage(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxxage'//'_htc'//zchar
            z2d(:,:) = sxxage(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'syyage'//'_htc'//zchar
            z2d(:,:) = syyage(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            znam = 'sxyage'//'_htc'//zchar
            z2d(:,:) = sxyage(:,:,jl)
            CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
         END DO

         CALL iom_rstput( iter, nitrst, numriw, 'sxopw ' ,  sxopw  )
         CALL iom_rstput( iter, nitrst, numriw, 'syopw ' ,  syopw  )
         CALL iom_rstput( iter, nitrst, numriw, 'sxxopw' ,  sxxopw )
         CALL iom_rstput( iter, nitrst, numriw, 'syyopw' ,  syyopw )
         CALL iom_rstput( iter, nitrst, numriw, 'sxyopw' ,  sxyopw )
         
         DO jl = 1, jpl 
            WRITE(zchar,'(I2.2)') jl
            DO jk = 1, nlay_i 
               WRITE(zchar1,'(I2.2)') jk
               znam = 'sxe'//'_il'//zchar1//'_htc'//zchar
               z2d(:,:) = sxe(:,:,jk,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'sye'//'_il'//zchar1//'_htc'//zchar
               z2d(:,:) = sye(:,:,jk,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'sxxe'//'_il'//zchar1//'_htc'//zchar
               z2d(:,:) = sxxe(:,:,jk,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'syye'//'_il'//zchar1//'_htc'//zchar
               z2d(:,:) = syye(:,:,jk,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'sxye'//'_il'//zchar1//'_htc'//zchar
               z2d(:,:) = sxye(:,:,jk,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            END DO
         END DO
         IF ( nn_pnd_scheme > 0 ) THEN
            DO jl = 1, jpl 
               WRITE(zchar,'(I2.2)') jl
               znam = 'sxap'//'_htc'//zchar
               z2d(:,:) = sxap(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'syap'//'_htc'//zchar
               z2d(:,:) = syap(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'sxxap'//'_htc'//zchar
               z2d(:,:) = sxxap(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'syyap'//'_htc'//zchar
               z2d(:,:) = syyap(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'sxyap'//'_htc'//zchar
               z2d(:,:) = sxyap(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
   
               znam = 'sxvp'//'_htc'//zchar
               z2d(:,:) = sxvp(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'syvp'//'_htc'//zchar
               z2d(:,:) = syvp(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'sxxvp'//'_htc'//zchar
               z2d(:,:) = sxxvp(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'syyvp'//'_htc'//zchar
               z2d(:,:) = syyvp(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
               znam = 'sxyvp'//'_htc'//zchar
               z2d(:,:) = sxyvp(:,:,jl)
               CALL iom_rstput( iter, nitrst, numriw, znam , z2d )
            END DO
         ENDIF
         !
      ENDIF
      !
   END SUBROUTINE adv_pra_rst

#else
   !!----------------------------------------------------------------------
   !!   Default option            Dummy module        NO ESIM sea-ice model
   !!----------------------------------------------------------------------
#endif

   !!======================================================================
END MODULE icedyn_adv_pra
